# -*- coding: utf-8 -*-
# Generated by Django 1.10.5 on 2017-02-22 23:24
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djcelery', '0003_auto_20170222_2304'),
    ]

    operations = [
        migrations.AlterField(
            model_name='periodictask',
            name='timezone',
            field=models.CharField(blank=True, default='UTC', max_length=128, null=True, verbose_name='timezone'),
        ),
    ]
