# -*- coding: utf-8 -*-
# Generated by Django 1.10.5 on 2017-02-22 23:04
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djcelery', '0002_crontabschedule_nowfun'),
    ]

    operations = [
        migrations.RenameField(
            model_name='periodictask',
            old_name='last_run_at',
            new_name='_last_run_at',
        ),
        migrations.AddField(
            model_name='periodictask',
            name='timezone',
            field=models.CharField(blank=True, max_length=128, null=True, verbose_name='timezone'),
        ),
    ]
